@@ -2,18 +2,138 @@
 # Author: Zhang
 #============================================================## 
 
-#============================================================## 
-## Set class
-#============================================================##
-setClass("MGLMsparsereg", representation(call = "function", data = "list", coefficients = "matrix", 
-    logL = "numeric", BIC = "numeric", AIC = "numeric", Dof = "numeric", iter = "numeric", 
-    maxlambda = "numeric", lambda = "numeric", distribution = "character", penalty = "character", 
-    Beta = "numeric"))
-
 ##============================================================## 
 ## Sparse reg function 
 ##============================================================##
-
+#' @title Fit multivariate GLM sparse regression
+#'
+#' @description Fit sparse regression in multivariate generalized linear models.
+#'
+#' @param formula an object of class \code{formula} (or one that can be coerced
+#'   to that class): a symbolic description of the model to be fitted.
+#'   The response has to be on the left hand side of ~. 
+#' @param data an optional data frame, list or environment (or object coercible by 
+#'   \code{as.data.frame} to a data frame) containing the variables in the model.
+#'   If not found in \code{data} when using function \code{MGLMsparsereg}, the variables 
+#'   are taken from \code{environment(formula)}, typically the environment from
+#'   which \code{MGLMsparsereg} is called.
+#' @param Y a matrix containing the multivariate categorical response data. 
+#'   Rows of the matrix represent observations, while columns are the different
+#'   categories.  Rows and columns of all zeros are automatically removed when
+#'   \code{dist="MN"}, \code{"DM"}, or \code{"GDM"}.  
+#' @param X design matrix (including intercept).  
+#'    Number of rows of the matrix should match that of \code{Y}.  
+#' @param dist a description of the error distribution to fit. See \code{\link{dist}} for details. 
+#' @param weight an optional vector of weights assigned to each row of the data. 
+#' 	Should be \code{NULL} or a numeric vector. Could be a variable from \code{data}, 
+#' 	or a variable from \code{environment(formula)} with the length equal to
+#' 	the number of rows of the data.
+#' 	If \code{weight=NULL}, equal weights of ones will be assigned.
+#' @param init an optional matrix of initial value of the parameter estimates.
+#' 	Should have the compatible dimension with the data. See \code{\link{dist}} for
+#' 	details of the dimensions in each distribution. 
+#' @param lambda penalty parameter. 
+#' @param penalty penalty type for the regularization term. Can be chosen from \code{"sweep"}, 
+#' 	\code{"group"}, or \code{"nuclear"}. See Details for the description of each penalty type.
+#' @param penidx a logical vector indicating the variables to be penalized. The default value is \code{rep(TRUE, p)}, which means all predictors are subject to regularization. If \code{X} contains intercept, use \code{penidx=c(FALSE,rep(TRUE,p-1))}.
+#' @param maxiters an optional numeric controlling the maximum number of iterations. The default
+#' 	value is maxiters=150.
+#' @param ridgedelta an optional numeric controlling the behavior of the Nesterov's accelerated proximal gradient method. The default value is \eqn{\frac{1}{pd}}{1/pd}.
+#' @param epsilon an optional numeric controlling the stopping criterion. The algorithm terminates when the relative change in the objective values of two successive iterates is less then \code{epsilon}.
+#'   	The default value is \code{epsilon=1e-5}.
+#' @param regBeta an optional logical variable used when running negative multinomial regression (\code{dist="NegMN"}).  
+#' 	\code{regBeta} controls whether to run regression on the over-dispersion parameter.
+#' 	The default is \code{regBeta=FALSE}.
+#' @param overdisp an optional numerical variable used only when fitting sparse negative multinomial 
+#' 	model \code{dist="NegMN"} and \code{regBeta=FALSE}.  \code{overdisp} gives the over dispersion value
+#' 	for all the observations.  The default value is estimated using negative-multinomial regression.  When \code{dist="MN", "DM", "GDM"} or \code{regBeta=TRUE}, the value of \code{overdisp} is ignored. 
+#'
+#' 	
+#' @details 
+#' In general, we consider regularization problem
+#' \deqn{
+#' \min_B h(B) = -l(B)+ J(B),
+#' }{
+#' min_B h(B) = -l(B)+ J(B),
+#' }
+#' where \eqn{l(B)} is the loglikelihood function and \eqn{J(B)} is the 
+#' regularization function.  
+#' 
+#' Sparsity in the individual elements of the parameter matrix \eqn{B} is achieved 
+#' by the lasso penalty (\code{dist="sweep"})
+#' \deqn{
+#' J(B) = \lambda \sum_{k\in penidx} \sum_{j=1}^d \|B_{kj}\|
+#' }{
+#' J(B) = \lambda \sum_{k in penidx} \sum_{j=1}^d ||B_{kj}||
+#' }
+#' 
+#' Sparsity in the rows of the regression parameter matrix \eqn{B} is achieved
+#' by the group penalty (\code{dist="group"})
+#' \deqn{
+#' J(B) = \lambda \sum_{k \in penidx} \|B_{k \cdot}\|_2,
+#' }{
+#' J(B) = \lambda \sum_{k in penidx} ||B_{k cdot}||_2,
+#' }
+#' where \eqn{\|v\|_2}{||v||_2} is the \eqn{l_2}{L2} norm of a vector \eqn{v}. In other words, 
+#' \eqn{\|B_{k\cdot}\|_2}{||B_{k cdot}||_2} is the \eqn{l_2}{L2} norm of the \eqn{k}-th row of the 
+#' parameter matrix \eqn{B}.
+#' 
+#' Sparsity in the rank of the parameter matrix \eqn{B} is achieved by the nuclear norm penalty (\code{dist="nuclear"})
+#' \deqn{
+#' J(B) = \lambda \|B\|_*= \lambda \sum_{i=1}^{min(p, d)} \sigma_i(B),
+#' }{
+#' J(B) = \lambda ||B||_{*}= \lambda \sum_{i=1}^{min(p, d)} \sigma_i(B),
+#' }
+#' where \eqn{\sigma_i(B)} are the singular values of the parameter matrix \eqn{B}. 
+#' The nuclear norm \eqn{\|B\|_*}{||B||_{*}} is a convex relaxation of \eqn{rank(B)=\|\sigma(B)\|_0}{rank(B)=||\sigma(B)||_0}.
+#' 
+#' See \code{\link{dist}} for details about distributions. 
+#' 
+#' 
+#' @return Returns an object of class \code{"MGLMsparsereg"}. An object of class \code{"MGLMsparsereg"} is a list containing at least the following components:  \itemize{
+#' \item{\code{coefficients}}{ the estimated matrix of regression coefficients.}
+#' \item{\code{logL}}{ the final loglikelihood value.}
+#' \item{\code{AIC}}{ Akaike information criterion.}
+#' \item{\code{BIC}}{ Bayesian information criterion.}
+#' \item{\code{Dof}}{ degrees of freedom of the estimated parameter.}
+#' \item{\code{iter}}{ number of iterations used. }
+#' \item{\code{maxlambda}}{ the maxmum tuning parameter such that 
+#' 	the estimated coefficients are not all zero.  This value is returned only
+#' 	when the tuning parameter \code{lambda} given to the function is large enough 
+#' 	such that all the parameter estimates are zero; otherwise, \code{maxlambda}
+#' 	is not computed.}
+#' \item{\code{call}}{ a matched call.}
+#' \item{\code{data}}{ the data used to fit the model: a list of the predictor matrix
+#' and the response matrix.}
+#' \item{\code{penalty}}{ the penalty chosen when running the penalized regression.}
+#' }
+#'
+#' @author Yiwen Zhang and Hua Zhou
+#'
+#' @examples 
+#' ## Generate Dirichlet Multinomial data
+#' dist <- "DM"
+#' n <- 100
+#' p <- 15
+#' d <- 5
+#' m <- runif(n, min=0, max=25) + 25
+#' set.seed(134)
+#' X <- matrix(rnorm(n*p),n, p)
+#' alpha <- matrix(0, p, d)
+#' alpha[c(1,3, 5), ] <- 1
+#' Alpha <- exp(X\%*\%alpha)
+#' Y <- rdirmn(size=m, alpha=Alpha)
+#' 
+#' ## Tuning
+#' ngridpt <- 10
+#' p <- ncol(X)
+#' d <- ncol(Y)
+#' pen <- 'nuclear'
+#' spfit <- MGLMsparsereg(formula=Y~0+X, dist=dist, lambda=Inf, penalty=pen)
+#' 
+#' @keywords regression 
+#'
+#' @export
 MGLMsparsereg <- function(formula, data, dist, lambda, penalty, weight, init, penidx, 
     maxiters = 150, ridgedelta, epsilon = 1e-05, regBeta = FALSE, overdisp) {
     call <- match.call()
@@ -69,24 +189,24 @@ MGLMsparsereg <- function(formula, data, dist, lambda, penalty, weight, init, pe
     est <- eval(call("MGLMsparsereg.fit", Y = Y, X = X, dist = dist, lambda = lambda, 
         penalty = penalty, weight = weight, init = init, penidx = penidx, maxiters = maxiters, 
         ridgedelta = ridgedelta, epsilon = epsilon, regBeta = regBeta, overdisp = overdisp))
-    est$call <- match.call()
-    est$data <- list(Y = Y, X = X)
-    est$distribution <- ifelse(dist == "MN", "Multinomial", ifelse(dist == "DM", 
-        "Dirichlet Multinomial", ifelse(dist == "GDM", "Generalized Dirichlet Multinomial", 
-            "Negative Multinomial")))
-    est$penalty <- ifelse(penalty == "group_row", "group", ifelse(penalty == "nuclear", 
-        "nuclear", "sweep"))
-    est$lambda <- lambda
-    class(est) <- "MGLMsparsereg"
-    return(est)
+    est@call <- match.call()
+    
+    ## ----------------------------------------## 
+    ## Set class
+    ## ----------------------------------------## 
+    return(est) 
+
 }
 
 ## ============================================================## 
 ## The MGLM sparse reg function 
 ## ============================================================##
+#' @rdname MGLMsparsereg
+#' @export 
 MGLMsparsereg.fit <- function(Y, X, dist, lambda, penalty, weight, init, penidx, 
     maxiters = 150, ridgedelta, epsilon = 1e-05, regBeta = FALSE, overdisp) {
     
+    tol <- 1e-8 
     d <- ncol(Y)
     m <- rowSums(Y)
     p <- ncol(X)
@@ -127,7 +247,7 @@ MGLMsparsereg.fit <- function(Y, X, dist, lambda, penalty, weight, init, penidx,
             pen <- matrix_threshold(X = Apen, lambda = ridgedelta * lambda, penalty = penalty)
             B[penidx, ] <- pen[[1]]
             penval <- pen[[2]]
-            if (all(abs(B[penidx, ]) == 0)) {
+            if (all(abs(B[penidx, ]) < tol)) {
                 if (penalty == "sweep") {
                   maxlambda <- max(abs(Apen))/ridgedelta
                 } else if (penalty == "group_row") {
@@ -141,7 +261,7 @@ MGLMsparsereg.fit <- function(Y, X, dist, lambda, penalty, weight, init, penidx,
                   penval <- 0
                 }
             } else {
-                maxlambda <- NULL
+                maxlambda <- numeric() # 12/1/17
             }
             objloss <- MGLM.loss(Y, X, B, dist, weight, regBeta, overdisp)
             loss.S2 <- objloss[[1]]
@@ -177,7 +297,7 @@ MGLMsparsereg.fit <- function(Y, X, dist, lambda, penalty, weight, init, penidx,
         Bpen <- B[penidx, ] else if (d == 2) 
         Bpen <- matrix(B[penidx, ], , 1)
     if (penalty == "sweep") {
-        Dof <- sum(penidx == FALSE) * ncol(B) + sum(Bpen != 0)
+        Dof <- sum(!penidx) * ncol(B) + sum(Bpen != 0)
     } else if (penalty == "group_row") {
         Dof <- sum(!penidx) * ncol(B) + sum(rowSums(Bpen^2) != 0) + sum((ncol(B) - 
             1) * rowSums(Bpen^2)/rowSums(Apen^2))
@@ -205,88 +325,34 @@ MGLMsparsereg.fit <- function(Y, X, dist, lambda, penalty, weight, init, penidx,
         B <- B[, c(outOrder, outOrder + d - 1)]
     }
     
-    return(list(coefficients = B, logL = logL, AIC = AIC, BIC = BIC, Dof = Dof, iter = niter, 
-        maxlambda = maxlambda))
+ 
+    distribution <- ifelse(dist == "MN", "Multinomial", ifelse(dist == "DM", 
+       "Dirichlet Multinomial", ifelse(dist == "GDM", "Generalized Dirichlet Multinomial", 
+                                       "Negative Multinomial")))
+    penalty_name <- ifelse(penalty == "group_row", "group", ifelse(penalty == "nuclear", 
+                                                                  "nuclear", "sweep"))
+    new("MGLMsparsereg", call = match.call(), data = list(Y = Y, X = X), 
+        coefficients = B, logL = logL, BIC = BIC, AIC = AIC, Dof = Dof, 
+        iter = niter, maxlambda = maxlambda, lambda = lambda, 
+        distribution = distribution, penalty = penalty_name) # , Beta = est$Beta) 
 }
 
 ## ============================================================================##
-## MGLM logss
+## MGLM loss
 ## ============================================================================##
 
 MGLM.loss <- function(Y, X, beta, dist, weight, regBeta = FALSE, Beta) {
-    N <- nrow(Y)
-    d <- ncol(Y)
-    p <- ncol(X)
-    m <- rowSums(Y)
-    if (missing(weight)) 
-        weight <- rep(1, N)
-    if (dist == "MN") {
-        P <- matrix(NA, N, d)
-        P[, d] <- rep(1, N)
-        P[, 1:(d - 1)] <- exp(X %*% beta)
-        P <- P/rowSums(P)
-        loss <- -sum(weight * dmn(Y, P))
-        kr1 <- Y[, -d] - P[, 1:(d - 1)] * m
-        if (d > 2) {
-            lossD1 <- -colSums(kr(kr1, X, weight))
-        } else if (d == 2) {
-            lossD1 <- -colSums(kr1 * weight * X)
-        }
-        lossD1 <- matrix(lossD1, p, (d - 1))
-    } else if (dist == "DM") {
-        alpha <- exp(X %*% beta)
-        loss <- -sum(weight * ddirm(Y, alpha))
-        tmpvector <- digamma(rowSums(alpha) + m) - digamma(rowSums(alpha))
-        tmpmatrix <- digamma(alpha + Y) - digamma(alpha)
-        dalpha <- tmpmatrix - tmpvector
-        lossD1 <- -rowSums(sapply(1:nrow(Y), function(i, A, B, w) return(w[i] * A[i, 
-            ] %x% B[i, ]), alpha * dalpha, X, weight))
-        lossD1 <- matrix(lossD1, p, d)
-    } else if (dist == "GDM") {
-        Ys <- t(apply(apply(apply(Y, 1, rev), 2, cumsum), 2, rev))
-        alpha <- exp(X %*% beta)
-        A <- alpha[, 1:(d - 1)]
-        B <- alpha[, d:(2 * (d - 1))]
-        loss <- -sum(weight * dgdirm(Y, A, B))
-        dalpha1 <- digamma(A + Y[, -d]) - digamma(A) - digamma(A + B + Ys[, -d]) + 
-            digamma(A + B)
-        dalpha2 <- digamma(B + Ys[, -1]) - digamma(B) - digamma(A + B + Ys[, -d]) + 
-            digamma(A + B)
-        dalpha <- cbind(dalpha1, dalpha2)
-        lossD1 <- -rowSums(sapply(1:nrow(Y), function(i, A, B, w) return(w[i] * A[i, 
-            ] %x% B[i, ]), alpha * dalpha, X, weight))
-        lossD1 <- matrix(lossD1, p, 2 * (d - 1))
-    } else if (dist == "NegMN") {
-        if (regBeta) {
-            P <- matrix(NA, N, (d + 1))
-            alpha <- exp(X %*% beta)
-            Beta <- alpha[, (d + 1)]
-            alpha_rowsums <- rowSums(alpha[, 1:d]) + 1
-            P[, (d + 1)] <- 1/alpha_rowsums
-            P[, 1:d] <- alpha[, 1:d] * P[, (d + 1)]
-            loss <- -sum(weight * dneg(Y, alpha[, 1:d], Beta))
-            deta <- matrix(0, nrow(Y), d + 1)
-            deta[, 1:d] <- Y - alpha[, 1:d] * Beta - P[, 1:d] * (m - (alpha_rowsums - 
-                1) * Beta)
-            deta[, (d + 1)] <- Beta * (digamma(Beta + m) - digamma(Beta) + log(P[, 
-                (d + 1)]))
-            lossD1 <- -rowSums(sapply(1:nrow(Y), function(i, A, B, w) return(w[i] * 
-                A[i, ] %x% B[i, ]), deta, X, weight))
-            lossD1 <- matrix(lossD1, p, (d + 1))
-        } else {
-            P <- matrix(NA, N, (d + 1))
-            alpha <- exp(X %*% beta)
-            alpha_rowsums <- rowSums(alpha) + 1
-            P[, (d + 1)] <- 1/alpha_rowsums
-            P[, 1:d] <- alpha[, 1:d] * P[, (d + 1)]
-            loss <- -sum(weight * dneg(Y, alpha, rep(Beta, N)))
-            deta <- Y - (Beta + m) * P[, 1:d]
-            lossD1 <- -rowSums(sapply(1:N, function(i, A, B, w) return(w[i] * A[i, 
-                ] %x% B[i, ]), deta, X, weight))
-            lossD1 <- matrix(lossD1, p, d)
-        }
-    }
-    return(list(loss, lossD1))
+	
+	 if (missing(weight)) 
+        weight <- rep(1, nrow(Y))
+        
+     losslist <- switch(dist,
+     					"MN"= DMD.MN.loss(Y, X, beta, dist, weight),
+     					"DM" = DMD.DM.loss(Y, X, beta, dist, weight),
+     					"GDM" = DMD.GDM.loss(Y, X, beta, dist, weight),
+     					"NegMN" = DMD.NegMN.loss(Y, X, beta, dist, weight, regBeta = FALSE, Beta))
+	
+     return(losslist)
 }
 
 ## ============================================================================##
